@@ -1,5 +1,4 @@
-__version__ = "2.1.0"
-
+__version__ = "3.7.1"
 
 import logging
 import logging.config
@@ -8,13 +7,16 @@ import os
 import yaml
 from dotenv import load_dotenv
 
-path = "zoish/config.yaml"
-DEFAULT_LEVEL = logging.INFO
+from .project_conf import ROOT_PROJECT
+
+PATH_TO_LOG_CONF = ROOT_PROJECT / "zoish" / "config.yaml"
+# DEFAULT_LEVEL in production env
+DEFAULT_LEVEL = logging.ERROR
 
 
-def log_setup(log_cfg_path=path):
+def log_setup(log_cfg_path=PATH_TO_LOG_CONF):
     try:
-        with open(path, "r") as f:
+        with open(PATH_TO_LOG_CONF, "r") as f:
             config = yaml.safe_load(f.read())
             logging.config.dictConfig(config)
             # set up logging configuration
